% This file demonstrates the population size estimation model described in
% [1]. Traces are first generated and then the contact matrix is
% computed. Afterward population size is estimated by using the
% arrival/departure times of the individuals, the arrival/departure
% time distribution of the individuals, and the approximated arrival/departure
% time distribution of individuals. Two population size estimators
% in [2] and [3] are also applied to the traces.
%
% Similar methodology is used in producing Figures 2 and 3, Tables 2 and 3, and
% Sections 5.1 and 5.2  of [1], and in producing Figures 3, 4 and 5 of [4].
%
% [1]: "Opportunistic Sampling for Joint Population Size and Density Estimation",
% F. M. Naini, O. Dousse, P. Thiran, and M. Vetterli, IEEE Transactions on Mobile
% Computing, Volume 14, Issue 12, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/
% [2]: Estimating population size via sample coverage for closed capture-recapture
% models, S. Lee and A. Chao, Biometrics, 1994
% [3]: Population estimation with performance guarantees, A. Orlitsky, N. Santhanam
% and K. Viswanathan, IEEE International Symposium on Information Theory (ISIT), 2007
% [4]: "Opportunistic Sampling for Joint Population Size and Density
% Estimation: Supplementary Notes", F. M. Naini, O. Dousse, P. Thiran,
% and M. Vetterli, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/

clear all;
close all;
clc;

%% selecting parameters
% Feel free to modify these values
N = 1000; % population size (i.e., number of individuals)
M = 10; % number of agents
alpha0 = 1.5; % parameter of the gamma distribution modeling the contact rates
beta0 = 1000; % parameter of the gamma distribution modeling the contact rates
eventDuration = 600; % duration of the experiment (e.g., event duration) or observation window

% optimization options
% by changing "off" to "iter", more details are shown on the console
options = optimset('GradObj', 'off', 'Display', 'off', 'MaxFunEvals', 300000, 'Algorithm', 'interior-point', 'MaxIter', 300000, 'TolX', 1e-12, 'TolCon', 1e-12, 'TolFun', 1e-9);

%% simulating agents' and individuals' contact traces
% In the following function, contTrace is a cell containing M matrices where
% each matrix corresponds to the traces of one agent.
% Each matrix has two columns where the first column
% is the ID of the individual contacted by the agent and the second column
% is the time of contact. The IDs of individuals are numbered from 1 to S
% where S <= N is the total number of discovered individuals. The time of contacts are
% between 0 and eventDuration.
% agentEntDepTimes is an Mx2 matrix where the first column is the arrival
% times of the agents to the event and the second column is the departure
% times of the agents from the event.
% allEntDepTimes is an Nx2 containing the arrival and departure times of all
% the N individuals to/from the event.
% discIndEntDepTimes is an Sx2 matrix containing the arrival and departure
% times of all the S discovered individuals to/from the event.
fprintf('\nGenerating traces:\n');
[contTrace, S, agentEntDepTimes, allEntDepTimes, discIndEntDepTimes] = traceSimulate1(alpha0, beta0, eventDuration, M, N);

%% obtaining the contact matrix from the agents' and individuals' contact traces
% In the following function, contMat is an SxM matrix where element ij is equal to the number of times
% that agent j contacts individual i, for 1 <= i <= S and 1 <= j <= M.
[contMat] = contMatCalc(contTrace, M, S);

%% displaying some properties of the traces
% plotting individuals' arrival/departure times -- scatter plot
figure, plot(allEntDepTimes(:, 1), allEntDepTimes(:, 2), '.', 'markersize', 5);
hold on;
xlim([0 eventDuration]);
ylim([0 eventDuration])
xlabel('Arrival time')
ylabel('Departure time')
title('Scatter plot of the arrival/departure times')
% plotting agents' arrival/departure times
plot(agentEntDepTimes(:, 1), agentEntDepTimes(:, 2), 'r*', 'markersize', 3);
legend('Individuals', 'Agents', 'Location', 'southeast');

% plotting individuals' arrival/departure times -- density function
[bandwidth, density, X, Y] = kde2d(allEntDepTimes, 2 ^ 8, [0 0], [eventDuration eventDuration]);
figure, imagesc(flipud(density));
cp = colormap('gray');
colormap(flipud(cp));
ax = gca;
ax.XTick = [0 64 128 192 256];
ax.XTickLabel = {'0' num2str(eventDuration / 4) num2str(eventDuration / 2) num2str(eventDuration * 3 / 4) num2str(eventDuration)};
ax.YTick = [0 64 128 192 256];
ax.YTickLabel = fliplr({'0' num2str(eventDuration / 4) num2str(eventDuration / 2) num2str(eventDuration * 3 / 4) num2str(eventDuration)});
xlabel('Arrival time');
ylabel('Departure time');
title('Joint arrival/departure time distribution of the individuals');
hold on;
plot(agentEntDepTimes(:, 1), agentEntDepTimes(:, 2), 'r*', 'markersize', 3);

% plotting the marginal distributions
[fArv, xiArv] = ksdensity(allEntDepTimes(:, 1), 'support', [- 1 eventDuration + 1], 'kernel', 'triangle');
[fDep, xiDep] = ksdensity(allEntDepTimes(:, 2), 'support', [- 1 eventDuration + 1], 'kernel', 'triangle');
[fDur, xiDur] = ksdensity(allEntDepTimes(:, 2) - allEntDepTimes(:, 1), 'support', [- 1 eventDuration + 1], 'kernel', 'triangle');
figure, subplot 311
plot(xiArv, fArv, 'markersize', 2);
title('Marginal distributions: Arrival times');
xlim([- 1 eventDuration + 1]);
subplot 312
plot(xiDep, fDep, 'markersize', 2);
title('Departure times');
xlim([- 1 eventDuration + 1]);
subplot 313
plot(xiDur, fDur, 'markersize', 2);
title('Duration of stay');
xlabel('Time');
xlim([- 1 eventDuration + 1]);

% plotting the evolution of the number of discovered individuals by agents,
% and the total number of discovered individuals and the duration of stay for
% each agent
displayTrace(contTrace, eventDuration, M, agentEntDepTimes);

% displaying on the console
fprintf('Number of individuals N: %d \n', N);
fprintf('Number of agents M: %d\n', M);
fprintf('Duration of event (observation window): %d\n', eventDuration);
fprintf('Parameters of the gamma distribution, alpha: %d, beta: %d\n', alpha0, beta0);
fprintf('Number of discovered individuals S: %d (%1.1d%% coverage)\n', S, S / N * 100);

%% Estimating the population size by using the actual arrival/departure times of the discovered individuals
fprintf('\nEstimating the population size by using the actual arrival/departure times of the discovered individuals:\n');
% estimating initial alpha0 and beta0 for optimization
[alphaStart, betaStart] = parameterStart1(contMat, agentEntDepTimes, allEntDepTimes);

% performing the optimization
funcOpt = @(parameters)PopEstLogLikeKnownTimes(parameters, contMat, discIndEntDepTimes, agentEntDepTimes, allEntDepTimes);
params1 = fmincon(funcOpt, [alphaStart betaStart], [], [], [], [], [0 0], [Inf Inf], [], options);
[~, NHat1] = funcOpt(params1);

% displaing the results
fprintf('Estimated population size: %d\n', NHat1);
fprintf('Estimated parameters of the gamma distribution, alpha: %d, beta: %d\n', params1(1), params1(2));

%% Estimating the population size by using the actual distribution of arrival/departure times of the individuals
fprintf('\nEstimating the population size by using the actual distribution of arrival/departure times of the individuals:\n');
funcOpt = @(parameters)PopEstLogLikeKnownDistribution(parameters, contMat, agentEntDepTimes, allEntDepTimes);

% performing the optimization
params2 = fmincon(funcOpt, [alphaStart betaStart], [], [], [], [], [0 0], [Inf Inf], [], options);
[~, NHat2] = funcOpt(params2);

fprintf('Estimated population size: %d\n', NHat2);
fprintf('Estimated parameters of the gamma distribution, alpha: %d, beta: %d\n', params2(1), params2(2));

%% Estimating the population size by using the approximated distribution of arrival/departure times of the individuals
% assuming all the individuals arrive at time 0 and depart at the end of the event
fprintf('\nEstimating the population size by using the approximated distribution of arrival/departure times of the individuals:');
fprintf('\nAssuming all the individuals arrive at time 0 and depart at the end of the event.\n');

% estimating initial alpha0 and beta0 for optimization
[alphaStart, betaStart] = parameterStart1(contMat, agentEntDepTimes, [0 eventDuration]);

funcOpt = @(parameters)PopEstLogLikeKnownDistribution(parameters, contMat, agentEntDepTimes, [0 eventDuration]);

% performing the optimization
params3 = fmincon(funcOpt, [alphaStart betaStart], [], [], [], [], [0 0], [Inf Inf], [], options);
[~, NHat3] = funcOpt(params3);

fprintf('Estimated population size: %d\n', NHat3);
fprintf('Estimated parameters of the gamma distribution, alpha: %d, beta: %d\n', params3(1), params3(2));

%% Estimating the population size by using the approximated distribution of arrival/departure times of the individuals
% Using Gaussian approximation of the individuals arrival/departure times distribution

fprintf('\nEstimating the population size by using the Gaussian approximation of the distribution of arrival/departure times of the individuals:\n');

% Gaussian approximation of the distribution
numSammples = S; % numSamples is the number of samples of the distribution over which
% we compute the expectation in the likelihood function.
allEntDepTimesGauEst(:, 1) = randn(numSammples, 1) * std(allEntDepTimes(:, 1)) + mean(allEntDepTimes(:, 1));
allEntDepTimesGauEst(:, 2) = randn(numSammples, 1) * std(allEntDepTimes(:, 2)) + mean(allEntDepTimes(:, 2));
allEntDepTimesGauEst(allEntDepTimesGauEst(:, 1) < 0, 1) = 0;
allEntDepTimesGauEst(allEntDepTimesGauEst(:, 2) > eventDuration, 2) = eventDuration;
allEntDepTimesGauEst(allEntDepTimesGauEst(:, 2) < 0, 2) = 0;
allEntDepTimesGauEst(allEntDepTimesGauEst(:, 1) > eventDuration, 1) = eventDuration;
allEntDepTimesGauEst = sort(allEntDepTimesGauEst, 2);

% estimating initial alpha0 and beta0 for optimization
[alphaStart, betaStart] = parameterStart1(contMat, agentEntDepTimes, allEntDepTimesGauEst);

funcOpt = @(parameters)PopEstLogLikeKnownDistribution(parameters, contMat, agentEntDepTimes, allEntDepTimesGauEst);

% performing the optimization
params4 = fmincon(funcOpt, [alphaStart betaStart], [], [], [], [], [0 0], [Inf Inf], [], options);
[~, NHat4] = funcOpt(params4);

fprintf('Estimated population size: %d\n', NHat4);
fprintf('Estimated parameters of the gamma distribution, alpha: %d, beta: %d\n', params4(1), params4(2));

%% Estimating the population size by other estimators in the literature
NHat5 = chao1(contMat);
fprintf('\nEstimated population size by the capture-recapture method in [2] is: %d\n', NHat5);

NHat6 = orlitsky1(sum(contMat, 2));
fprintf('\nEstimated population size by the estimator in [1] is: %d\n', NHat6);