% This function gets as input the contact traces between the individuals
% and agents and a new subset of agents and a new observation window and
% then discards all the contact traces that do not belong to the new subset
% of agents and the new observation window.
%
% Input:
% contTrace is a cell containing M matrices where each matrix corresponds to the
% traces of one agent. Each matrix has two columns where the first column
% is the ID of the individual contacted by the agent and the second column
% is the time of contact. The IDs of individuals are numbered from 1 to S.
% The rows of the matrix are sorted according to the time of contact.
% agentEntDepTimes is an Mx2 matrix where the first column is the arrival
% times of the agents to the event and the second column is the departure
% times of the agents from the event.
% allEntDepTimes is a two-column matrix containing samples of the
% arrival/departure times of the individuals to/from the event. Note that
% these samples could represent the actual distribution or an approximate
% distribution of the arrival/departure times.
% discIndEntDepTimes is an Sx2 matrix containing the arrival and departure
% times of all the S discovered individuals to/from the event.
% agentsSubsets is a vector containing the subset of agents from 1 to M.
% Only the traces of these agents are considered and the traces of the rest
% of the agents are discarded.
% timeDuration is a two element vector defining the new observation window.
% Only the traces belonging to this window are considered and the rest of
% the traces are discarded.
%
% Output:
% contTraceShrink is the rebuild of contTrace when the traces belonging to
% only the agentsSubsets and timeDuration are taken into account.
% SShrink is the rebuild of SShrink when the traces belonging to
% only the agentsSubsets and timeDuration are taken into account.
% agentEntDepTimesShrink is the rebuild of agentEntDepTimesShrink when the traces belonging to
% only the agentsSubsets and timeDuration are taken into account.
% allEntDepTimesShrink is the rebuild of allEntDepTimesShrink when the traces belonging to
% only the agentsSubsets and timeDuration are taken into account.
% discIndEntDepTimesShrink is the rebuild of discIndEntDepTimesShrink when the traces belonging to
% only the agentsSubsets and timeDuration are taken into account.

function [contTraceShrink, SShrink, agentEntDepTimesShrink, allEntDepTimesShrink, discIndEntDepTimesShrink] = traceShrinker(contTrace, agentEntDepTimes, allEntDepTimes, discIndEntDepTimes, agentsSubsets, timeDuration)

% choosing the subset of agents
contTraceShrink = contTrace(agentsSubsets);

% chosing the subset of event time duration
for agentIter = 1:length(agentsSubsets)
    agentTrace = contTraceShrink{agentIter};
 
    % considering contacts that occured during the subset of event time duration
    agentTrace = agentTrace(agentTrace(:, 2) >= timeDuration(1) & agentTrace(:, 2) <= timeDuration(2), :);
 
    % shifting the time of contact
    agentTrace(:, 2) = agentTrace(:, 2) - timeDuration(1);
    contTraceShrink{agentIter} = agentTrace;
end

% modifying the agents' arrival departure times
agentEntDepTimesShrink = agentEntDepTimes(agentsSubsets, :);
agentEntDepTimesShrink(:, 1) = max(agentEntDepTimesShrink(:, 1), timeDuration(1));
agentEntDepTimesShrink(:, 2) = min(agentEntDepTimesShrink(:, 2), timeDuration(2));
agentEntDepTimesShrink = agentEntDepTimesShrink - timeDuration(1);

% finding the IDs of the discovered inidividuals by the subset of agents
% and subset of event time duration
isDisovered = zeros(size(allEntDepTimes, 1), 1);

for agentIter = 1:length(agentsSubsets)
    agentTrace = contTraceShrink{agentIter};
    % soring the individuals that were discovered by the subset of the
    % agents and the subset of the event time duration
    isDisovered(agentTrace(:, 1)) = 1;
end

SShrink = sum(isDisovered == 1); % calculating SShrink

% modifying the arrrival/departure times of the discovered individuals
discIndEntDepTimesShrink = discIndEntDepTimes(isDisovered == 1, :);
discIndEntDepTimesShrink(:, 1) = max(discIndEntDepTimesShrink(:, 1), timeDuration(1));
discIndEntDepTimesShrink(:, 2) = min(discIndEntDepTimesShrink(:, 2), timeDuration(2));
discIndEntDepTimesShrink = discIndEntDepTimesShrink - timeDuration(1);

% modifying the IDs discovered individuals
newIndice = cumsum(isDisovered);
for agentIter = 1:length(agentsSubsets)
    agentTrace = contTraceShrink{agentIter};
    agentTrace(:, 1) = newIndice(agentTrace(:, 1));
    contTraceShrink{agentIter} = agentTrace;
end

% modifying the arrival/departure times of all individuals
%finding the overlap time between individuals' arrival departure times and
%the new event time duration
overlap = min(allEntDepTimes(:, 2), timeDuration(2)) - max(allEntDepTimes(:, 1), timeDuration(1));
allEntDepTimesShrink = allEntDepTimes(overlap > 0, :);
allEntDepTimesShrink(:, 1) = max(allEntDepTimesShrink(:, 1), timeDuration(1));
allEntDepTimesShrink(:, 2) = min(allEntDepTimesShrink(:, 2), timeDuration(2));
allEntDepTimesShrink = allEntDepTimesShrink - timeDuration(1);