% This function simulates contact traces between agents and individuals
% according to the joint population size and density estimation model in [1].
%
% Input:
% N is the population size (i.e., number of individuals)
% M is the number of agents
% K is the number of locations
% alpha0 and beta0 are parameters of the gamma distribution modeling the
% contact rates. alpha0 is equal to the summation of the alpha parameter of
% the location-dependent gamma distribution of all the locations, i.e.,
% alpha_0=sum(alpha_l), for l=1,...,K.
% eventDuration is the duration of the experiment (e.g., event duration)
%
% Output:
% contTrace is a cell containing M matrices where each matrix corresponds to the
% traces of one agent.  Each matrix has three columns where the first column
% is the ID of the individual contacted by the agent, where the second column
% is the time of contact and where the third columns is the location where the contact
% occurred. The IDs of individuals are numbered from 1 to S
% where S <= N is the total number of discovered individuals. The time of contacts are
% between 0 and eventDuration. The locations of contacts are between 1 and K.
% The rows of the matrix are sorted according to the time of contact.
% agentEntDepTimes is an Mx2 matrix where the first column is the arrival
% times of the agents to the event and the second column is the departure
% times of the agents from the event.
% agentTraj is a cell containing M matrices where each Matrix represents
% the trajectory of an agent. Each matrix has three columns where the first
% column is the arrival time of the agent to a location, where the second
% column is the departure time of the agent from that location, and where the
% third column is the ID of that location.
% allEntDepTimes is an Nx2 containing the arrival and departure times of all
% the N individuals to/from the event.
% discIndEntDepTimes is an Sx2 matrix containing the arrival and departure
% times of all the S discovered individuals to/from the event.
% K is the number of locations. This value might be different than the value
% passed as input.
% PiVec is a vector containing the density values of each location. This
% vector sums to one and is based on the definition in Section 6.1 of [1].
%
% [1]: "Opportunistic Sampling for Joint Population Size and Density Estimation",
% F. M. Naini, O. Dousse, P. Thiran, and M. Vetterli, IEEE Transactions on Mobile
% Computing, Volume 14, Issue 12, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/

function [contTrace, S, agentEntDepTimes, agentTraj, allEntDepTimes, discIndEntDepTimes, K, PiVec] = traceSimulate2(alpha0, beta0, eventDuration, M, N, K)
%% simulating the spatial density ground truth
% feel free to modify this part
xGrids = fix(sqrt(K / 2) * 2); % number of locations in the x-axis
yGrids = fix(sqrt(K / 2)); % number of locations in the y-axis
K = xGrids * yGrids; % re-computing the number of locations

% simulating the density
[X1, X2] = meshgrid([0:xGrids - 1], [0:yGrids - 1]);
sigmaScale = K ^ (0.5);
miu1 = [xGrids / 4 yGrids / 2];
Sigma1 = [1 0; 0 1] * sigmaScale;
miu2 = [xGrids * 3 / 4 yGrids / 4];
Sigma2 = [.25 .3; .3 1] * sigmaScale / 2;
miu3 = [xGrids * 3 / 4 yGrids * 3 / 4];
Sigma3 = [1 .3; .3 .25] * sigmaScale / 2;
% we use three different density peaks
dens1 = mvnpdf([X1(:) X2(:)], miu1, Sigma1);
dens2 = mvnpdf([X1(:) X2(:)], miu2, Sigma2);
dens3 = mvnpdf([X1(:) X2(:)], miu3, Sigma3);

% normalizing the density vector
PiVec = dens1 + 0.25 * dens2 + 0.125 * dens3;
PiVec = PiVec / sum(PiVec);
%PiVec = reshape(PiVec, yGrids, xGrids);

%% simulating individuals' arrival departure times
% feel free to modify this part
allEntDepTimes = zeros(N, 2);
allEntDepTimes(:, 1) = exprnd(eventDuration / 5, N, 1);
allEntDepTimes(allEntDepTimes(:, 1) > eventDuration * 0.95, 1) = eventDuration * 0.95;
allEntDepTimes(:, 2) = allEntDepTimes(:, 1) + randn(N, 1) * eventDuration / 4 + 1 / 2 * eventDuration;
allEntDepTimes(allEntDepTimes(:, 2) > eventDuration, 2) = eventDuration;
allEntDepTimes = sort(allEntDepTimes, 2);
allEntDepTimes(allEntDepTimes(:, 1) < 0, 1) = 0;

%% simulating agents' arrival departure times
% feel free to modify this part
agentEntDepTimes = zeros(M, 2);
agentEntDepTimes(:, 1) = rand(M, 1) * eventDuration / 4;
agentEntDepTimes(:, 2) = eventDuration * 3 / 4 + rand(M, 1) * eventDuration / 4;

%% simulating agents' visit to different locations
agentTraj = cell(M, 1);
for agentIter = 1:M
    % number of locations that the agent visits
    visitNum = 1 + poissrnd(K);
    % time instances when the agent moves from one location to another
    visitEpoch = [agentEntDepTimes(agentIter, 1), rand([1, visitNum - 1]) * (agentEntDepTimes(agentIter, 2) - agentEntDepTimes(agentIter, 1)) + agentEntDepTimes(agentIter, 1), agentEntDepTimes(agentIter, 2)];
    visitEpoch = sort(visitEpoch);
    % choosing the visited locations
    traj = [];
 
    prevLoc = 0;
    for epochIter = 1:visitNum
        %choosing agent's next location uniformly at random
        nextLoc = randi(K, 1);
        %choosing agent's next location with the same density as the individuals
        %nextLoc=find(mnrnd(1, PiVec));
     
        if prevLoc ~= nextLoc
            %agent's next location is different than the current location
            %so will add a new row to the trajectory
            traj = [traj; [visitEpoch(epochIter), visitEpoch(epochIter + 1), nextLoc]];
            prevLoc = nextLoc;
        else
            %agent's next location is the same as the current location
            %so will not add a new row to the trajectory
            traj(end, 2) = visitEpoch(epochIter + 1);
        end
    end
 
    agentTraj{agentIter} = traj;
end

%% simulating the location-dependent contact rates
contRates = zeros(N, K);
for locIter = 1:K
    contRates(:, locIter) = gamrnd(alpha0 * PiVec(locIter), 1 / beta0, [1, N]);
end

%% simulating the contacts
isDisovered = zeros(N, 1); %to identify the individuals that have been discovered by the agents

contTrace = cell(M, 1); %buffer
for agentIter = 1:M
    % iteration on every agent. Building the trace for this agent.
    agentTrace = [];
 
    traj = agentTraj{agentIter};
 
    for locIter = 1:size(traj, 1)
        visitedLoc = traj(locIter, 3);
        locEntTime = traj(locIter, 1);
        locDepTime = traj(locIter, 2);
        for indIter = 1:N
            % iteration on every individual. Building the trace for this individual and the agent.
            % first computing the duration of time that both the individual and the
            % agent are inside the event
            commonOverlapTime = max(min(allEntDepTimes(indIter, 2), locDepTime) - max(allEntDepTimes(indIter, 1), locEntTime), 0);
            if commonOverlapTime > 0
                % if commonOverlapTime>0 then there is possibility of contact
                % between the two
                contNum = poissrnd(commonOverlapTime * contRates(indIter, visitedLoc)); % simulating the number of contacts
                if contNum > 0
                    % if we have contacts, then we simulate the time of contacts between the individual and the agent
                    contTimes = rand(contNum, 1) * commonOverlapTime + max(allEntDepTimes(indIter, 1), locEntTime);
                    isDisovered(indIter) = 1; % indicating that this individual is discovered
                    agentTrace = [agentTrace; [ones(length(contTimes), 1) * indIter contTimes ones(length(contTimes), 1) * visitedLoc]];
                end
            end
        end
    end
 
    if ~ isempty(agentTrace)
        % sorting the agent's trace based on time
        [~, index] = sort(agentTrace(:, 2));
        agentTrace = agentTrace(index, :);
    end
    contTrace{agentIter} = agentTrace; % saving the agent's trace
 
end

S = sum(isDisovered == 1); % calculating S
discIndEntDepTimes = allEntDepTimes(isDisovered == 1, :); % calculating the arrrival/departure times of the discovered individuals

newIndice = cumsum(isDisovered);
% renaming the IDs of the discovered individuals to 1--S
for agentIter = 1:M
    agentTrace = contTrace{agentIter, 1};
    agentTrace(:, 1) = newIndice(agentTrace(:, 1));
    contTrace{agentIter, 1} = agentTrace;
end